@@ -10,7 +10,7 @@ Getting Connection Information
 To get connected item to a handle::
 
     c = canvas.get_connection(handle)
-    if c != None:
+    if c is not None:
         print c.connected
         print c.port
         print c.constraint
@@ -700,7 +700,7 @@ class Canvas(object):
             self._post_update_items(dirty_items, cr)
 
         except Exception, e:
-            logging.error('Error while updating canvas', e)
+            logging.error('Error while updating canvas', exc_info=e)
 
         assert len(self._dirty_items) == 0 and len(self._dirty_matrix_items) == 0, \
                 'dirty: %s; matrix: %s' % (self._dirty_items, self._dirty_matrix_items)
