@@ -2,13 +2,13 @@
 Custom decorators.
 """
 
-__version__ = "$Revision: 2292 $"
-# $HeadURL: http://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.3.6/gaphas/decorators.py $
+__version__ = "$Revision$"
+# $HeadURL$
 
+import threading
 import gobject
 from gobject import PRIORITY_HIGH, PRIORITY_HIGH_IDLE, PRIORITY_DEFAULT, \
         PRIORITY_DEFAULT_IDLE, PRIORITY_LOW
-import operator
 
 
 DEBUG_ASYNC = False
@@ -40,13 +40,23 @@ class async(object):
     ...     def a(self):
     ...         print 'idle-a', gobject.main_depth()
     
-    Methods can also set sinle mode to True (the method is only scheduled one).
+    Methods can also set single mode to True (the method is only scheduled one).
 
     >>> class B(object):
     ...     @async(single=True)
     ...     def b(self):
     ...         print 'idle-b', gobject.main_depth()
 
+    Also a timeout property can be provided:
+
+    >>> class C(object):
+    ...     @async(timeout=50)
+    ...     def c1(self):
+    ...         print 'idle-c1', gobject.main_depth()
+    ...     @async(single=True, timeout=60)
+    ...     def c2(self):
+    ...         print 'idle-c2', gobject.main_depth()
+
     This is a helper function used to test classes A and B from within the GTK+
     main loop:
 
@@ -54,6 +64,11 @@ class async(object):
     ...     print 'before'
     ...     a = A()
     ...     b = B()
+    ...     c = C()
+    ...     c.c1()
+    ...     c.c1()
+    ...     c.c2()
+    ...     c.c2()
     ...     a.a()
     ...     b.b()
     ...     a.a()
@@ -72,17 +87,35 @@ class async(object):
     idle-a 1
     idle-a 1
     idle-b 1
+    idle-c1 1
+    idle-c1 1
+    idle-c2 1
 
     As you can see, although ``b.b()`` has been called three times, it's only
     executed once.
     """
 
-    def __init__(self, single=False, priority=gobject.PRIORITY_DEFAULT):
+    def __init__(self, single=False, timeout=0, priority=gobject.PRIORITY_DEFAULT):
         self.single = single
+        self.timeout = timeout
         self.priority = priority
 
+    def source(self, func):
+        timeout = self.timeout
+        if timeout > 0:
+            s = gobject.Timeout(timeout)
+        else:
+            s = gobject.Idle()
+        s.set_callback(func)
+        s.priority = self.priority
+        return s
+
     def __call__(self, func):
+        async_id = '_async_id_%s' % func.__name__
+        source = self.source
+
         def wrapper(*args, **kwargs):
+            global getattr, setattr, delattr
             # execute directly if we're not in the main loop.
             if gobject.main_depth() == 0:
                 return func(*args, **kwargs)
@@ -90,12 +123,12 @@ class async(object):
                 def async_wrapper():
                     if DEBUG_ASYNC: print 'async:', func, args, kwargs
                     func(*args, **kwargs)
-                gobject.idle_add(async_wrapper, priority=self.priority)
+                source(async_wrapper).attach()
             else:
+                # Idle handlers should be registered per instance
                 holder = args[0]
                 try:
-                    f = operator.attrgetter('_async_id_%s' % func.__name__)
-                    if f(holder):
+                    if getattr(holder, async_id):
                         return
                 except AttributeError, e:
                     def async_wrapper():
@@ -103,11 +136,10 @@ class async(object):
                         try:
                             func(*args, **kwargs)
                         finally:
-                            holder.__delattr__('_async_id_%s' % func.__name__)
+                            delattr(holder, async_id)
                         return False
 
-                    holder.__setattr__('_async_id_%s' % func.__name__,
-                        gobject.idle_add(async_wrapper, priority=self.priority))
+                    setattr(holder, async_id, source(async_wrapper).attach())
         return wrapper
 
 
@@ -125,21 +157,16 @@ def nonrecursive(func):
     >>> A().a()
     1
     """
+    m = threading.Lock()
     def wrapper(*args, **kwargs):
         """
         Decorate function with a mutex that prohibits recursice execution.
         """
-        try:
-            if func._executing:
-                return
-        except AttributeError:
-            # _executed not present
-            pass
-        try:
-            func._executing = True
-            return func(*args, **kwargs)
-        finally:
-            del func._executing
+        if m.acquire(False):
+            try:
+                return func(*args, **kwargs)
+            finally:
+                m.release()
     return wrapper
 
 
@@ -177,8 +204,4 @@ class recursive(object):
         return wrapper
 
 
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
-
-# vim:sw=4:et
+# vim:sw=4:et:ai
