@@ -3,7 +3,7 @@ The painter module provides different painters for parts of the canvas.
 
 Painters can be swapped in and out.
 
-Each painter takes are of a layer in the canvas (such as grid, items
+Each painter takes care of a layer in the canvas (such as grid, items
 and handles).
 """
 
@@ -15,6 +15,8 @@ from cairo import Matrix, ANTIALIAS_NONE, LINE_JOIN_ROUND
 from gaphas.canvas import Context
 from gaphas.geometry import Rectangle
 from gaphas.item import Line
+from gaphas.aspect import PaintFocused
+
 
 DEBUG_DRAW_BOUNDING_BOX = False
 
@@ -27,6 +29,12 @@ class Painter(object):
     Painter interface.
     """
 
+    def __init__(self, view=None):
+        self.view = view
+
+    def set_view(self, view):
+        self.view = view
+
     def paint(self, context):
         """
         Do the paint action (called from the View).
@@ -40,14 +48,22 @@ class PainterChain(Painter):
     like ToolChain.
     """
 
-    def __init__(self):
+    def __init__(self, view=None):
+        super(PainterChain, self).__init__(view)
         self._painters = []
 
+    def set_view(self, view):
+        self.view = view
+        for painter in self._painters:
+            painter.set_view(self.view)
+
     def append(self, painter):
         """
         Add a painter to the list of painters.
         """
         self._painters.append(painter)
+        painter.set_view(self.view)
+        return self
 
     def prepend(self, painter):
         """
@@ -66,7 +82,7 @@ class PainterChain(Painter):
 class DrawContext(Context):
     """
     Special context for draw()'ing the item. The draw-context contains
-    stuff like the view, the cairo context and properties like selected and
+    stuff like the cairo context and properties like selected and
     focused.
     """
 
@@ -80,14 +96,14 @@ class ItemPainter(Painter):
 
     draw_all = False
 
-    def _draw_item(self, item, view, cairo, area=None):
+    def _draw_item(self, item, cairo, area=None):
+        view = self.view
         cairo.save()
         try:
             cairo.set_matrix(view.matrix)
             cairo.transform(view.canvas.get_matrix_i2c(item))
 
             item.draw(DrawContext(painter=self,
-                                  view=view,
                                   cairo=cairo,
                                   _area=area,
                                   _item=item,
@@ -100,17 +116,18 @@ class ItemPainter(Painter):
         finally:
             cairo.restore()
 
-    def _draw_items(self, items, view, cairo, area=None):
+    def _draw_items(self, items, cairo, area=None):
         """
         Draw the items.
         """
         for item in items:
             #if not area or area - view.get_item_bounding_box(item):
-            self._draw_item(item, view, cairo, area=area)
+            self._draw_item(item, cairo, area=area)
             if DEBUG_DRAW_BOUNDING_BOX:
-                self._draw_bounds(item, view, cairo)
+                self._draw_bounds(item, cairo)
 
-    def _draw_bounds(self, item, view, cairo):
+    def _draw_bounds(self, item, cairo):
+        view = self.view
         try:
             b = view.get_item_bounding_box(item)
         except KeyError:
@@ -126,12 +143,9 @@ class ItemPainter(Painter):
 
     def paint(self, context):
         cairo = context.cairo
-        view = context.view
-        area = context.area
-        items = context.items
         cairo.set_tolerance(TOLERANCE)
         cairo.set_line_join(LINE_JOIN_ROUND)
-        self._draw_items(items, view, cairo, area)
+        self._draw_items(context.items, cairo, context.area)
 
 
 class CairoBoundingBoxContext(object):
@@ -237,12 +251,13 @@ class BoundingBoxPainter(ItemPainter):
 
     draw_all = True
 
-    def _draw_item(self, item, view, cairo, area=None):
+    def _draw_item(self, item, cairo, area=None):
         cairo = CairoBoundingBoxContext(cairo)
-        super(BoundingBoxPainter, self)._draw_item(item, view, cairo)
+        super(BoundingBoxPainter, self)._draw_item(item, cairo)
         bounds = cairo.get_bounds()
 
         # Update bounding box with handles.
+        view = self.view
         i2v = view.get_matrix_i2v(item).transform_point
         for h in item.handles():
             cx, cy = i2v(*h.pos)
@@ -252,18 +267,15 @@ class BoundingBoxPainter(ItemPainter):
         view.set_item_bounding_box(item, bounds)
 
 
-    def _draw_items(self, items, view, cairo, area=None):
+    def _draw_items(self, items, cairo, area=None):
         """
         Draw the items.
         """
         for item in items:
-            self._draw_item(item, view, cairo)
+            self._draw_item(item, cairo)
 
     def paint(self, context):
-        cairo = context.cairo
-        view = context.view
-        items = context.items
-        self._draw_items(items, view, cairo)
+        self._draw_items(context.items, context.cairo)
 
 
 class HandlePainter(Painter):
@@ -271,11 +283,12 @@ class HandlePainter(Painter):
     Draw handles of items that are marked as selected in the view.
     """
 
-    def _draw_handles(self, item, view, cairo, opacity=None, inner=False):
+    def _draw_handles(self, item, cairo, opacity=None, inner=False):
         """
         Draw handles for an item.
         The handles are drawn in non-antialiased mode for clearity.
         """
+        view = self.view
         cairo.save()
         i2v = view.get_matrix_i2v(item)
         if not opacity:
@@ -316,19 +329,19 @@ class HandlePainter(Painter):
         cairo.restore()
 
     def paint(self, context):
-        view = context.view
+        view = self.view
         canvas = view.canvas
         cairo = context.cairo
         # Order matters here:
         for item in canvas.sort(view.selected_items):
-            self._draw_handles(item, view, cairo)
+            self._draw_handles(item, cairo)
         # Draw nice opaque handles when hovering an item:
         item = view.hovered_item
         if item and item not in view.selected_items:
-            self._draw_handles(item, view, cairo, opacity=.25)
+            self._draw_handles(item, cairo, opacity=.25)
         item = view.dropzone_item
         if item and item not in view.selected_items:
-            self._draw_handles(item, view, cairo, opacity=.25, inner=True)
+            self._draw_handles(item, cairo, opacity=.25, inner=True)
 
 
 class ToolPainter(Painter):
@@ -338,7 +351,7 @@ class ToolPainter(Painter):
     """
 
     def paint(self, context):
-        view = context.view
+        view = self.view
         cairo = context.cairo
         if view.tool:
             cairo.save()
@@ -346,53 +359,28 @@ class ToolPainter(Painter):
             view.tool.draw(context)
             cairo.restore()
 
-# Move this to tool.
-class LineSegmentPainter(Painter):
+class FocusedItemPainter(Painter):
     """
-    This painter draws pseudo-hanldes on gaphas.item.Line objects. Each
-    line can be split by dragging those points, which will result in
-    a new handle.
-
-    ConnectHandleTool take care of performing the user
-    interaction required for this feature.
+    This painter allows for drawing on top off all other layers for the
+    focused item.
     """
 
     def paint(self, context):
-        view = context.view
+        view = self.view
         item = view.hovered_item
-        if item and item is view.focused_item and isinstance(item, Line):
-            cr = context.cairo
-            h = item.handles()
-            for h1, h2 in zip(h[:-1], h[1:]):
-                p1, p2 = h1.pos, h2.pos
-                cx = (p1.x + p2.x) / 2
-                cy = (p1.y + p2.y) / 2
-                cr.save()
-                cr.identity_matrix()
-                m = Matrix(*view.get_matrix_i2v(item))
-
-                cr.set_antialias(ANTIALIAS_NONE)
-                cr.translate(*m.transform_point(cx, cy))
-                cr.rectangle(-3, -3, 6, 6)
-                cr.set_source_rgba(0, 0.5, 0, .4)
-                cr.fill_preserve()
-                cr.set_source_rgba(.25, .25, .25, .6)
-                cr.set_line_width(1)
-                cr.stroke()
-                cr.restore()
+        if item and item is view.focused_item:
+            PaintFocused(item, view).paint(context)
 
 
-
-def DefaultPainter():
+def DefaultPainter(view=None):
     """
     Default painter, containing item, handle and tool painters.
     """
-    chain = PainterChain()
-    chain.append(ItemPainter())
-    chain.append(HandlePainter())
-    chain.append(LineSegmentPainter())
-    chain.append(ToolPainter())
-    return chain
+    return PainterChain(view). \
+        append(ItemPainter()). \
+        append(HandlePainter()). \
+        append(FocusedItemPainter()). \
+        append(ToolPainter())
 
 
-# vim: sw=4:et:
+# vim: sw=4:et:ai
