@@ -18,10 +18,8 @@ For this to work the revert_handler has to be added to the observers set::
 
 """
 
-__version__ = "$Revision: 2240 $"
-# $HeadURL: http://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.3.6/gaphas/state.py $
-
 import types, inspect
+import threading
 from decorator import decorator
 
 # This string is added to each docstring in order to denote is's observed
@@ -44,6 +42,9 @@ subscribers = set()
 # Subscribe to low-level change events:
 observers = set()
 
+# Perform locking (should be per thread?).
+mutex = threading.Lock()
+
 def observed(func):
     """
     Simple observer, dispatches events to functions registered in the observers
@@ -58,62 +59,20 @@ def observed(func):
     """
     def wrapper(func, *args, **kwargs):
         o = func.__observer__
+        acquired = mutex.acquire(False)
         try:
-            if o.__dispatched__:
+            if acquired:
                 dispatch((o, args, kwargs), queue=observers)
-        except AttributeError:
-            pass
-        return func(*args, **kwargs)
+            return func(*args, **kwargs)
+        finally:
+            if acquired:
+                mutex.release()
     dec = decorator(wrapper)(func)
     
     func.__observer__ = dec
-    if DISPATCH_BY_DEFAULT:
-        dec.__dispatched__ = True
     return dec
 
 
-def enable_dispatching(func, enable=True):
-    """
-    Enable/disable dispatching for a specific function.
-
-    >>> @observed
-    ... def callme():
-    ...     pass
-    >>> def handler(event):
-    ...   print 'event'
-    >>> observers.add(handler)
-
-    By default dispatching is enabled (set DISPATCH_BY_DEFAULT=False to
-    disable this behavior).
-
-    >>> callme()
-    event
-    >>> enable_dispatching(callme, False)
-    >>> callme()
-
-    ... and enable it again:
-
-    >>> enable_dispatching(callme)
-    >>> callme()
-    event
-    >>> observers.remove(handler)
-    """
-    func = getfunction(func)
-    if enable:
-        func.__dispatched__ = True
-    else:
-        try:
-            del func.__dispatched__
-        except AttributeError:
-            pass
-
-def disable_dispatching(func, disable=True):
-    """
-    Inverse operation of enable_dispatching()
-    """
-    enable_dispatching(func, not disable)
-
-
 def dispatch(event, queue):
     """
     Dispatch an event to a queue of event handlers.
@@ -237,10 +196,6 @@ def revert_handler(event):
 
     >>> reversible_pair(SList.add, SList.remove, \
         bind1={'before': lambda self, node: self.l[self.l.index(node)+1] })
-    >>> _reverse[SList.add.im_func] # doctest: +ELLIPSIS
-    (<function remove at 0x...>, (['self', 'node'], None, None, None), {})
-    >>> _reverse[SList.remove.im_func] # doctest: +ELLIPSIS
-    (<function add at 0x...>, (['self', 'node', 'before'], None, None, (None,)), {'before': <function <lambda> at ...>})
     >>> def handler(event):
     ...     print 'handle', event
     >>> subscribers.add(handler)
@@ -279,8 +234,8 @@ def revert_handler(event):
     kwargs = {}
     for arg in argnames:
         kwargs[arg] = kw.get(arg)
-    dispatch((reverse, kwargs), queue=subscribers)
 
+    dispatch((reverse, kwargs), queue=subscribers)
 
 def saveapply(func, kw):
     """
@@ -307,8 +262,4 @@ def getfunction(func):
     return func
 
 
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
-
 # vim:sw=4:et:ai
