@@ -8,7 +8,7 @@ from gaphas.view import View, GtkView
 from gaphas.canvas import Canvas, Context
 from gaphas.item import Line
 from gaphas.examples import Box
-from gaphas.tool import ToolContext, HoverTool
+from gaphas.tool import HoverTool
 
 
 class ViewTestCase(unittest.TestCase):
@@ -33,7 +33,7 @@ class ViewTestCase(unittest.TestCase):
         line = Line()
         line.fyzzyness = 1
         line.handles()[1].pos = (30, 30)
-        line.split_segment(0, 3)
+        #line.split_segment(0, 3)
         line.matrix.translate(30, 60)
         canvas.add(line)
 
@@ -79,8 +79,8 @@ class ViewTestCase(unittest.TestCase):
         assert len(view._qtree._ids) == 1
         assert not view._qtree._bucket.bounds == (0, 0, 0, 0), view._qtree._bucket.bounds
 
-        assert view.get_item_at_point(10, 10) is box
-        assert view.get_item_at_point(60, 10) is None
+        assert view.get_item_at_point((10, 10)) is box
+        assert view.get_item_at_point((60, 10)) is None
 
         window.destroy()
 
@@ -186,7 +186,28 @@ class ViewTestCase(unittest.TestCase):
         assert not box._matrix_v2i.has_key(view)
         
 
-        
+    def test_scroll_adjustments_signal(self):
+        def handler(self, hadj, vadj):
+            self.handled = True
+
+        sc = gtk.ScrolledWindow()
+        view = GtkView(Canvas())
+        view.connect('set-scroll-adjustments', handler)
+        sc.add(view)
+
+        assert view.handled
+
+
+    def test_scroll_adjustments(self):
+        sc = gtk.ScrolledWindow()
+        view = GtkView(Canvas())
+        sc.add(view)
+
+        print sc.get_hadjustment(), view.hadjustment
+        assert sc.get_hadjustment() is view.hadjustment
+        assert sc.get_vadjustment() is view.vadjustment
+
+
 if __name__ == '__main__':
     unittest.main()
 
