@@ -42,7 +42,7 @@ from gaphas.canvas import Context
 from gaphas.geometry import Rectangle
 from gaphas.geometry import distance_point_point_fast, distance_line_point
 from gaphas.item import Line
-from gaphas.aspect import Selection, InMotion, \
+from gaphas.aspect import Finder, Selection, InMotion, \
         HandleFinder, HandleSelection, HandleInMotion, \
         Connector
 
@@ -233,12 +233,7 @@ class HoverTool(Tool):
     def on_motion_notify(self, event):
         view = self.view
         pos = event.x, event.y
-        ## TODO: to aspect
-        item, handle = view.get_handle_at_point(pos)
-        if item:
-            view.hovered_item = item
-        else:
-            view.hovered_item = view.get_item_at_point(pos)
+        view.hovered_item = Finder(view).get_item_at_point(pos)
 
 
 class ItemTool(Tool):
@@ -784,11 +779,11 @@ class ConnectHandleTool(HandleTool):
 
 
 
-def DefaultTool():
+def DefaultTool(view=None):
     """
     The default tool chain build from HoverTool, ItemTool and HandleTool.
     """
-    chain = ToolChain(). \
+    return ToolChain(view). \
         append(HoverTool()). \
         append(ConnectHandleTool()). \
         append(PanTool()). \
@@ -796,7 +791,6 @@ def DefaultTool():
         append(ItemTool()). \
         append(TextEditTool()). \
         append(RubberbandTool())
-    return chain
 
 
 # vim: sw=4:et:ai
