@@ -3,18 +3,17 @@ Simple class containing the tree structure for the canvas items.
 
 """
 
-__version__ = "$Revision: 1790 $"
-# $HeadURL: http://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.3.6/gaphas/tree.py $
+__version__ = "$Revision$"
+# $HeadURL$
 
-import operator
-from state import observed, reversible_method, \
-        reversible_pair, disable_dispatching
+from operator import attrgetter
 
 
 class Tree(object):
     """
-    A Tree structure.
-    None is the root node.
+    A Tree structure. Nodes are stores in a depth-first order.
+    
+    ``None`` is the root node.
 
     @invariant: len(self._children) == len(self._nodes) + 1
     """
@@ -125,7 +124,7 @@ class Tree(object):
         >>> tree.get_children('n1')
         ['n2']
         >>> tree.get_all_children('n1') # doctest: +ELLIPSIS
-        <generator object at 0x...>
+        <generator object get_all_children at 0x...>
         >>> list(tree.get_all_children('n1'))
         ['n2', 'n3']
         """
@@ -146,7 +145,7 @@ class Tree(object):
         >>> tree.get_parent('n3')
         'n2'
         >>> tree.get_ancestors('n3') # doctest: +ELLIPSIS
-        <generator object at 0x...>
+        <generator object get_ancestors at 0x...>
         >>> list(tree.get_ancestors('n3'))
         ['n2', 'n1']
         >>> list(tree.get_ancestors('n1'))
@@ -157,42 +156,119 @@ class Tree(object):
             yield parent
             parent = self.get_parent(parent)
 
-    def _add_to_nodes(self, node, parent):
+    def index_nodes(self, index_key):
         """
-        Called only from add()
+        Provide each item in the tree with an index attribute. This makes
+        for fast sorting of items.
+
+        >>> class A(object):
+        ...     def __init__(self, n):
+        ...         self.n = n
+        ...     def __repr__(self):
+        ...         return self.n
+        >>> t = Tree()
+        >>> a = A('a')
+        >>> t.add(a)
+        >>> t.add(A('b'))
+        >>> t.add(A('c'), parent=a)
+        >>> t.nodes
+        [a, c, b]
+        >>> t.index_nodes('my_key')
+        >>> t.nodes[0].my_key, t.nodes[1].my_key, t.nodes[2].my_key
+        (0, 1, 2)
+
+        For sorting, see ``sort()``.
+        """
+        nodes = self.nodes
+        lnodes = len(nodes)
+        map(setattr, nodes, [index_key] * lnodes, xrange(lnodes))
+
+    def sort(self, nodes, index_key, reverse=False):
+        """
+        Sort a set (or list) of nodes.
+        
+        >>> class A(object):
+        ...     def __init__(self, n):
+        ...         self.n = n
+        ...     def __repr__(self):
+        ...         return self.n
+        >>> t = Tree()
+        >>> a = A('a')
+        >>> t.add(a)
+        >>> t.add(A('b'))
+        >>> t.add(A('c'), parent=a)
+        >>> t.nodes    # the series from Tree.index_nodes
+        [a, c, b]
+        >>> t.index_nodes('my_key')
+        >>> selection = (t.nodes[2], t.nodes[1])
+        >>> t.sort(selection, index_key='my_key')
+        [c, b]
+        """
+        if index_key:
+            return sorted(nodes, key=attrgetter(index_key), reverse=reverse)
+        else:
+            raise NotImplemented('index_key should be provided.')
+
+    def _add_to_nodes(self, node, parent, index=None):
+        """
+        Helper method to place nodes on the right location in the nodes list
+        Called only from add() and reparent()
         """
         nodes = self._nodes
-        if parent:
-            try:
-                next_uncle = self.get_next_sibling(parent)
-            except IndexError:
-                # parent has no younger brothers..
-                # place it before the next uncle of grant_parent:
-                self._add_to_nodes(node, self.get_parent(parent))
+        siblings = self._children[parent]
+        try:
+            atnode = siblings[index]
+        except (TypeError, IndexError):
+            index = len(siblings)
+            #self._add_to_nodes(node, parent)
+            if parent:
+                try:
+                    next_uncle = self.get_next_sibling(parent)
+                except IndexError:
+                    # parent has no younger brothers..
+                    # place it before the next uncle of grant_parent:
+                    return self._add_to_nodes(node, self.get_parent(parent))
+                else:
+                    nodes.insert(nodes.index(next_uncle), node)
             else:
-                nodes.insert(nodes.index(next_uncle), node)
+                # append to root node:
+                nodes.append(node)
         else:
-            # append to root node:
-            nodes.append(node)
+            nodes.insert(nodes.index(atnode), node)
 
-    @observed
-    def add(self, node, parent=None):
+
+    def _add(self, node, parent=None, index=None):
+        """
+        Helper method for both add() and reparent().
+        """
+        assert node not in self._nodes
+
+        siblings = self._children[parent]
+
+        self._add_to_nodes(node, parent, index)
+        
+        # Fix parent-child and child-parent relationship
+        try:
+            siblings.insert(index, node)
+        except TypeError:
+            siblings.append(node)
+
+        # Create new entry for it's own children:
+        if parent:
+            self._parents[node] = parent
+
+
+    def add(self, node, parent=None, index=None):
         """
         Add node to the tree. parent is the parent node, which may
         be None if the item should be added to the root item.
 
         For usage, see the unit tests.
         """
-        assert not self._children.get(node)
-        siblings = self._children[parent]
-        self._add_to_nodes(node, parent)
-        siblings.append(node)
-        # Create new entry for it's own children:
+        self._add(node, parent, index)
         self._children[node] = []
-        if parent:
-            self._parents[node] = parent
 
-    @observed
+
     def _remove(self, node):
         # Remove from parent item
         self.get_siblings(node).remove(node)
@@ -227,8 +303,7 @@ class Tree(object):
         for c in self._children[node]:
             self._reparent_nodes(c, node)
         
-    @observed
-    def reparent(self, node, parent):
+    def reparent(self, node, parent, index=None):
         """
         Set new parent for a ``node``. ``Parent`` can be ``None``, indicating
         it's added to the top.
@@ -260,35 +335,21 @@ class Tree(object):
         >>> tree.nodes
         ['n4', 'n1', 'n3', 'n2']
         """
-        # Add to new parent's children:
-        self.get_siblings(node).remove(node)
-        self._children[parent].append(node)
-        self._parents[node] = parent
-        
-        # reorganize nodes
-        self._reparent_nodes(node, parent)
+        if parent is self.get_parent(node):
+            return
+
+        # Remove all node references:
+        old_parent = self.get_parent(node)
+        self._children[old_parent].remove(node)
+        self._nodes.remove(node)
+        if old_parent:
+            del self._parents[node]
+
+        self._add(node, parent, index)
+
+        # reorganize children in nodes list
+        for c in self._children[node]:
+            self._reparent_nodes(c, node)
 
 
-    reversible_pair(add, _remove,
-            bind1={'parent': lambda self, node: self.get_parent(node) })
-
-    reversible_method(reparent, reverse=reparent,
-            bind={'parent': lambda self, node: self.get_parent(node) })
-
-    # Disable add/remove by default, since they are handled by canvas.Canvas
-    disable_dispatching(add)
-    disable_dispatching(_remove)
-
-
-__test__ = {
-    'Tree.add': Tree.add,
-    'Tree.remove': Tree.remove,
-    'Tree.reparent': Tree.reparent,
-    }
-
-
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
-
-# vi:sw=4:et
+# vi: sw=4:et:ai
