@@ -31,7 +31,8 @@ class View(object):
 
     def __init__(self, canvas=None):
         self._matrix = Matrix()
-        self._painter = DefaultPainter()
+        self._painter = DefaultPainter(self)
+        self._bounding_box_painter = BoundingBoxPainter(self)
 
         # Handling selections.
         ### TODO: Move this to a context?
@@ -86,11 +87,7 @@ class View(object):
 
     def select_item(self, item):
         """
-        Select an item. This adds @item to the set of selected items. Do::
-
-            del view.selected_items
-
-        to clear the selected items list.
+        Select an item. This adds @item to the set of selected items.
         """
         self.queue_draw_item(item)
         if item not in self._selected_items:
@@ -204,12 +201,25 @@ class View(object):
         Set the painter to use. Painters should implement painter.Painter.
         """
         self._painter = painter
+        painter.set_view(self)
         self.emit('painter-changed')
 
 
     painter = property(lambda s: s._painter, _set_painter)
 
 
+    def _set_bounding_box_painter(self, painter):
+        """
+        Set the painter to use for bounding box calculations.
+        """
+        self._bounding_box_painter = painter
+        painter.set_view(self)
+        self.emit('painter-changed')
+
+
+    bounding_box_painter = property(lambda s: s._bounding_box_painter, _set_bounding_box_painter)
+
+
     def get_item_at_point(self, pos, selected=True):
         """
         Return the topmost item located at ``pos`` (x, y).
@@ -389,22 +399,21 @@ class View(object):
         Update the bounding boxes of the canvas items for this view, in 
         canvas coordinates.
         """
-        painter = BoundingBoxPainter()
+        painter = self._bounding_box_painter
         if items is None:
             items = self.canvas.get_all_items()
 
         # The painter calls set_item_bounding_box() for each rendered item.
-        painter.paint(Context(view=self,
-                              cairo=cr,
-                              items=items))
+        painter.paint(Context(cairo=cr,
+                              items=items,
+                              area=None))
 
         # Update the view's bounding box with the rest of the items
         self._bounds = Rectangle(*self._qtree.soft_bounds)
 
 
     def paint(self, cr):
-        self._painter.paint(Context(view=self,
-                                    cairo=cr,
+        self._painter.paint(Context(cairo=cr,
                                     items=self.canvas.get_all_items(),
                                     area=None))
 
@@ -608,7 +617,7 @@ class GtkView(gtk.DrawingArea, View):
         c = Rectangle(*self._qtree.soft_bounds)
 
         # view limits
-        v = Rectangle(0, 0, self.allocation.width, self.allocation.height)
+        v = Rectangle(0, 0, allocation.width, allocation.height)
 
         # union of these limits gives scrollbar limits
         if v in c:
@@ -623,7 +632,7 @@ class GtkView(gtk.DrawingArea, View):
         hadjustment.upper, vadjustment.upper = u.x1, u.y1
 
         # set page size
-        aw, ah = self.allocation.width, self.allocation.height
+        aw, ah = allocation.width, allocation.height
         hadjustment.page_size = aw
         vadjustment.page_size = ah
 
@@ -679,6 +688,7 @@ class GtkView(gtk.DrawingArea, View):
         a = self.allocation
         super(GtkView, self).queue_draw_area(0, 0, a.width, a.height)
 
+
     def request_update(self, items, matrix_only_items=(), removed_items=()):
         """
         Request update for items. Items will get a full update treatment, while
@@ -781,6 +791,9 @@ class GtkView(gtk.DrawingArea, View):
     def do_realize(self):
         gtk.DrawingArea.do_realize(self)
 
+        # Ensure updates are propagated
+        self._canvas.register_view(self)
+
         if self._canvas:
             self.request_update(self._canvas.get_all_items())
 
@@ -789,12 +802,13 @@ class GtkView(gtk.DrawingArea, View):
             # Although Item._matrix_{i2v|v2i} keys are automatically removed
             # (weak refs), better do it explicitly to be sure.
             self._clear_matrices()
-            self.canvas = None
         self._qtree.clear()
 
         self._dirty_items.clear()
         self._dirty_matrix_items.clear()
 
+        self._canvas.unregister_view(self)
+
         gtk.DrawingArea.do_unrealize(self)
 
     def do_expose_event(self, event):
@@ -813,8 +827,7 @@ class GtkView(gtk.DrawingArea, View):
         cr.clip()
 
         area = Rectangle(x, y, width=w, height=h)
-        self._painter.paint(Context(view=self,
-                                    cairo=cr,
+        self._painter.paint(Context(cairo=cr,
                                     items=self.get_items_in_rectangle(area),
                                     area=area))
 
